uniform Image terrain;
uniform vec2 res;

#ifdef VERTEX
vec4 position(mat4 transform_projection, vec4 vertex_position)
{
    // The order of operations matters when doing matrix multiplication.
    return transform_projection * vertex_position;
}
#endif

#ifdef PIXEL
vec4 effect(vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords)
{
    //get data
    vec2 step = 1.0/res;
    vec4 data = Texel(tex, texture_coords); //r = height, g = flow_x, b = flow_y
    float ground = Texel(terrain, texture_coords).r;
    vec4 data1 = Texel(tex, texture_coords + vec2(step.x, 0));
    float ground1 = Texel(terrain, texture_coords + vec2(step.x, 0)).r;
    vec4 data2 = Texel(tex, texture_coords + vec2(0, step.y));
    float ground2 = Texel(terrain, texture_coords + vec2(0, step.y)).r;

    //calculate normals
    vec2 xVec = vec2(step.x, 0);
	vec2 yVec = vec2(0, step.y);
	vec3 tanX = vec3(xVec, ground1-ground);
	vec3 tanY = vec3(yVec, ground2-ground);
	vec3 ground_nor = cross(tanX, tanY);
    ground_nor.z *= 64;
    ground_nor = normalize(ground_nor);
    tanX = vec3(xVec, ground1+data1.r-ground-data.r);
	tanY = vec3(yVec, ground2+data2.r-ground-data.r);
    vec3 water_nor = cross(tanX, tanY);
    water_nor.z *= 16;
    water_nor = normalize(water_nor);

    //shading
    vec3 lightDir = normalize(vec3(-1,-1,1));
	float groundLight = clamp(dot(lightDir, ground_nor), 0, 1);
    groundLight = 0.7 * groundLight + 0.3 - clamp(0.5 * data1.r, 0, 1);
    vec4 groundColor = vec4(0.52, 0.46, 0.35, 1);
    vec4 waterColor = vec4(0.19, 0.63, 0.95, 1);
    float waterLight = clamp(dot(lightDir, water_nor), 0, 1);
    waterLight = 0.7 * waterLight + 0.3;
    float blendfactor = 0.4 * clamp(data1.r * 10000, 0, 1) + data1.r * 2;

    return mix(groundColor*groundLight, waterColor*waterLight, clamp(blendfactor, 0, 1));
}
#endif